@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Quick Controls module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -43,10 +35,14 @@
 #include <qquickitem.h>
 #include <qcoreapplication.h>
 #include <qqmlengine.h>
+#include <qlibrary.h>
 #include <qdir.h>
 #include <QTouchDevice>
 #include <QGuiApplication>
 #include <QStyleHints>
+#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_NO_SDK)
+#include <private/qjnihelpers_p.h>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -56,6 +52,11 @@ static QString defaultStyleName()
 #if defined(QT_WIDGETS_LIB) && !defined(Q_OS_IOS) && !defined(Q_OS_ANDROID) && !defined(Q_OS_BLACKBERRY) && !defined(Q_OS_QNX) && !defined(Q_OS_WINRT)
     if (QCoreApplication::instance()->inherits("QApplication"))
         return QLatin1String("Desktop");
+#elif defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_NO_SDK)
+    if (QtAndroidPrivate::androidSdkVersion() >= 11)
+        return QLatin1String("Android");
+#elif defined(Q_OS_IOS)
+    return QLatin1String("iOS");
 #endif
     return QLatin1String("Base");
 }
@@ -86,6 +87,15 @@ bool QQuickControlSettings::hasTouchScreen() const
 #endif
 }
 
+bool QQuickControlSettings::isMobile() const
+{
+#if defined(Q_OS_IOS) || defined(Q_OS_ANDROID) || defined(Q_OS_BLACKBERRY) || defined(Q_OS_QNX) || defined(Q_OS_WINRT)
+    return true;
+#else
+    return false;
+#endif
+}
+
 static QString styleImportPath(QQmlEngine *engine, const QString &styleName)
 {
     QString path = qgetenv("QT_QUICK_CONTROLS_STYLE");
@@ -119,11 +129,29 @@ QQuickControlSettings::QQuickControlSettings(QQmlEngine *engine)
     if (fromResource(path))
         path = path.remove(0, 3); // remove qrc from the path
 
-    if (!QDir(path).exists()) {
+    QDir dir(path);
+    if (!dir.exists()) {
         QString unknownStyle = m_name;
         m_name = defaultStyleName();
         m_path = styleImportPath(engine, m_name);
         qWarning() << "WARNING: Cannot find style" << unknownStyle << "- fallback:" << styleFilePath();
+    } else {
+        typedef bool (*StyleInitFunc)();
+        typedef const char *(*StylePathFunc)();
+
+        foreach (const QString &fileName, dir.entryList()) {
+            if (QLibrary::isLibrary(fileName)) {
+                QLibrary lib(dir.absoluteFilePath(fileName));
+                StyleInitFunc initFunc = (StyleInitFunc) lib.resolve("qt_quick_controls_style_init");
+                if (initFunc)
+                    initFunc();
+                StylePathFunc pathFunc = (StylePathFunc) lib.resolve("qt_quick_controls_style_path");
+                if (pathFunc)
+                    m_path = QString::fromLocal8Bit(pathFunc());
+                if (initFunc || pathFunc)
+                    break;
+            }
+        }
     }
 
     connect(this, SIGNAL(styleNameChanged()), SIGNAL(styleChanged()));
